@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2003,2005 by Solar Designer
+ * Copyright (c) 1996-2003,2005,2006,2009 by Solar Designer
  */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@ int rec_version = 0;
 int rec_argc = 0;
 char **rec_argv;
 unsigned int rec_check;
+int rec_restoring_now = 0;
 
 static int rec_fd;
 static FILE *rec_file = NULL;
@@ -137,24 +138,26 @@ void rec_save(void)
 #endif
 }
 
-void rec_done(int aborted)
+void rec_done(int save)
 {
 	if (!rec_file) return;
 
-	if (aborted) rec_save();
+	if (save)
+		rec_save();
+	else
+		log_flush();
 
 	if (fclose(rec_file)) pexit("fclose");
 	rec_file = NULL;
 
-	if (!aborted && (!status.pass || status.pass == 3))
-	if (unlink(path_expand(rec_name)))
+	if (!save && unlink(path_expand(rec_name)))
 		pexit("unlink: %s", path_expand(rec_name));
 }
 
 static void rec_format_error(char *fn)
 {
-	if (ferror(rec_file))
-		pexit(fn);
+	if (fn && errno && ferror(rec_file))
+		pexit("%s", fn);
 	else {
 		fprintf(stderr, "Incorrect crash recovery file format: %s\n",
 			path_expand(rec_name));
@@ -188,12 +191,15 @@ void rec_restore_args(int lock)
 	if (!strcmp(line, RECOVERY_V3)) rec_version = 3; else
 	if (!strcmp(line, RECOVERY_V2)) rec_version = 2; else
 	if (!strcmp(line, RECOVERY_V1)) rec_version = 1; else
-	if (strcmp(line, RECOVERY_V0)) rec_format_error("fgets");
+	if (strcmp(line, RECOVERY_V0)) rec_format_error(NULL);
 
-	if (fscanf(rec_file, "%d\n", &argc) != 1) rec_format_error("fscanf");
+	if (fscanf(rec_file, "%d\n", &argc) != 1)
+		rec_format_error("fscanf");
+	if (argc < 2)
+		rec_format_error(NULL);
 	argv = mem_alloc_tiny(sizeof(char *) * (argc + 1), MEM_ALIGN_WORD);
 
-	argv[0] = "";
+	argv[0] = "john";
 
 	for (index = 1; index < argc; index++)
 	if (fgetl(line, sizeof(line), rec_file))
@@ -204,7 +210,7 @@ void rec_restore_args(int lock)
 	argv[argc] = NULL;
 
 	save_rec_name = rec_name;
-	opt_init(argc, argv);
+	opt_init(argv[0], argc, argv);
 	rec_name = save_rec_name;
 
 	if (fscanf(rec_file, "%u\n%u\n%x\n%x\n",
@@ -220,12 +226,16 @@ void rec_restore_args(int lock)
 	} else
 	if (fscanf(rec_file, "%d\n%d\n", &status.pass, &status.progress) != 2)
 		rec_format_error("fscanf");
+	if (status.pass < 0 || status.pass > 3)
+		rec_format_error(NULL);
 
 	if (rec_version < 3)
 		rec_check = 0;
 	else
 	if (fscanf(rec_file, "%x\n", &rec_check) != 1)
 		rec_format_error("fscanf");
+
+	rec_restoring_now = 1;
 }
 
 void rec_restore_mode(int (*restore_mode)(FILE *file))
@@ -237,4 +247,6 @@ void rec_restore_mode(int (*restore_mode)(FILE *file))
 
 	if (fclose(rec_file)) pexit("fclose");
 	rec_file = NULL;
+
+	rec_restoring_now = 0;
 }
